@@ -198,7 +198,7 @@ def fromul4(template, variables="variables", indent=0):
 		else:
 			result.append(scriptlet("\n%s%s\n" % ("\t"*indent, content)))
 
-	loopcounter = 0 # Used to number loop iterators
+	varcounter = 0 # Used to number loop iterators and local templates
 	result = xsc.Frag()
 
 	make_scriptlet("//@@@ BEGIN template source")
@@ -213,6 +213,7 @@ def fromul4(template, variables="variables", indent=0):
 	for i in xrange(10):
 		make_scriptlet("Object r%d = null;" % i)
 
+	defs = []
 	lastloc = None
 	for opcode in template.opcodes:
 		if opcode.code is not None and opcode.location is not lastloc:
@@ -287,38 +288,56 @@ def fromul4(template, variables="variables", indent=0):
 		elif opcode.code == "printx":
 			make_scriptlet("out.write(com.livinglogic.ul4.Utils.xmlescape(org.apache.commons.lang.ObjectUtils.toString(r%d)));" % opcode.r1)
 		elif opcode.code == "for":
-			loopcounter += 1
-			make_scriptlet("for (java.util.Iterator iterator%d = com.livinglogic.ul4.Utils.iterator(r%d); iterator%d.hasNext();)" % (loopcounter, opcode.r2, loopcounter))
+			varcounter += 1
+			make_scriptlet("for (java.util.Iterator iterator%d = com.livinglogic.ul4.Utils.iterator(r%d); iterator%d.hasNext();)" % (varcounter, opcode.r2, varcounter))
 			make_scriptlet("{")
 			indent += 1
-			make_scriptlet("r%d = iterator%d.next();" % (opcode.r1, loopcounter))
+			make_scriptlet("r%d = iterator%d.next();" % (opcode.r1, varcounter))
 		elif opcode.code == "endfor":
 			indent -= 1
 			make_scriptlet("}")
+		elif opcode.code == "def":
+			varcounter += 1
+			make_scriptlet("com.livinglogic.ul4.JSPTemplate template%d = new com.livinglogic.ul4.JSPTemplate()" % varcounter)
+			make_scriptlet("{")
+			indent += 1
+			make_scriptlet("public void execute(Writer out, Map variables) throws java.io.IOException")
+			indent += 1
+			make_scriptlet("{")
+			indent += 1
+			defs.append((opcode.arg, variables))
+			variables = "variables"
+		elif opcode.code == "enddef":
+			indent -= 1
+			make_scriptlet("}")
+			indent -= 1
+			make_scriptlet("};")
+			(arg, variables) = defs.pop()
+			make_scriptlet("%s.put(%s, template%d);" % (variables, _string(arg), varcounter))
 		elif opcode.code == "break":
 			make_scriptlet("break;")
 		elif opcode.code == "continue":
 			make_scriptlet("continue;")
 		elif opcode.code == "not":
-			make_scriptlet("r%d = com.livinglogic.ul4.Utils.getBool(r%d) ? Boolean.FALSE : Boolean.TRUE;" % (opcode.r1, opcode.r2))
+			make_scriptlet("r%d = !com.livinglogic.ul4.Utils.getBool(r%d);" % (opcode.r1, opcode.r2))
 		elif opcode.code == "neg":
 			make_scriptlet("r%d = com.livinglogic.ul4.Utils.neg(r%d);" % (opcode.r1, opcode.r2))
 		elif opcode.code == "contains":
-			make_scriptlet("r%d = com.livinglogic.ul4.Utils.contains(r%d, r%d) ? Boolean.TRUE : Boolean.FALSE;" % (opcode.r1, opcode.r2, opcode.r3))
+			make_scriptlet("r%d = com.livinglogic.ul4.Utils.contains(r%d, r%d);" % (opcode.r1, opcode.r2, opcode.r3))
 		elif opcode.code == "notcontains":
-			make_scriptlet("r%d = com.livinglogic.ul4.Utils.contains(r%d, r%d) ? Boolean.FALSE : Boolean.TRUE;" % (opcode.r1, opcode.r2, opcode.r3))
+			make_scriptlet("r%d = !com.livinglogic.ul4.Utils.contains(r%d, r%d);" % (opcode.r1, opcode.r2, opcode.r3))
 		elif opcode.code == "eq":
-			make_scriptlet("r%d = org.apache.commons.lang.ObjectUtils.equals(r%d, r%d) ? Boolean.TRUE : Boolean.FALSE;" % (opcode.r1, opcode.r2, opcode.r3))
+			make_scriptlet("r%d = org.apache.commons.lang.ObjectUtils.equals(r%d, r%d);" % (opcode.r1, opcode.r2, opcode.r3))
 		elif opcode.code == "ne":
-			make_scriptlet("r%d = org.apache.commons.lang.ObjectUtils.equals(r%d, r%d) ? Boolean.FALSE : Boolean.TRUE;" % (opcode.r1, opcode.r2, opcode.r3))
+			make_scriptlet("r%d = !org.apache.commons.lang.ObjectUtils.equals(r%d, r%d);" % (opcode.r1, opcode.r2, opcode.r3))
 		elif opcode.code == "lt":
-			make_scriptlet("r%d = com.livinglogic.ul4.Utils.lt(r%d, r%d) ? Boolean.TRUE : Boolean.FALSE;" % (opcode.r1, opcode.r2, opcode.r3))
+			make_scriptlet("r%d = com.livinglogic.ul4.Utils.lt(r%d, r%d);" % (opcode.r1, opcode.r2, opcode.r3))
 		elif opcode.code == "le":
-			make_scriptlet("r%d = com.livinglogic.ul4.Utils.le(r%d, r%d) ? Boolean.TRUE : Boolean.FALSE;" % (opcode.r1, opcode.r2, opcode.r3))
+			make_scriptlet("r%d = com.livinglogic.ul4.Utils.le(r%d, r%d);" % (opcode.r1, opcode.r2, opcode.r3))
 		elif opcode.code == "gt":
-			make_scriptlet("r%d = com.livinglogic.ul4.Utils.le(r%d, r%d) ? Boolean.FALSE : Boolean.TRUE;" % (opcode.r1, opcode.r2, opcode.r3))
+			make_scriptlet("r%d = !com.livinglogic.ul4.Utils.le(r%d, r%d);" % (opcode.r1, opcode.r2, opcode.r3))
 		elif opcode.code == "ge":
-			make_scriptlet("r%d = com.livinglogic.ul4.Utils.lt(r%d, r%d) ? Boolean.FALSE : Boolean.TRUE;" % (opcode.r1, opcode.r2, opcode.r3))
+			make_scriptlet("r%d = !com.livinglogic.ul4.Utils.lt(r%d, r%d);" % (opcode.r1, opcode.r2, opcode.r3))
 		elif opcode.code == "add":
 			make_scriptlet("r%d = com.livinglogic.ul4.Utils.add(r%d, r%d);" % (opcode.r1, opcode.r2, opcode.r3))
 		elif opcode.code == "sub":
@@ -353,30 +372,34 @@ def fromul4(template, variables="variables", indent=0):
 				make_scriptlet("r%d = com.livinglogic.ul4.Utils.repr(r%d);" % (opcode.r1, opcode.r2))
 			elif opcode.arg == "int":
 				make_scriptlet("r%d = com.livinglogic.ul4.Utils.toInteger(r%d);" % (opcode.r1, opcode.r2))
+			elif opcode.arg == "float":
+				make_scriptlet("r%d = com.livinglogic.ul4.Utils.toFloat(r%d);" % (opcode.r1, opcode.r2))
 			elif opcode.arg == "bool":
-				make_scriptlet("r%d = com.livinglogic.ul4.Utils.getBool(r%d) ? Boolean.TRUE : Boolean.FALSE;" % (opcode.r1, opcode.r2))
+				make_scriptlet("r%d = com.livinglogic.ul4.Utils.getBool(r%d);" % (opcode.r1, opcode.r2))
 			elif opcode.arg == "len":
 				make_scriptlet("r%d = com.livinglogic.ul4.Utils.length(r%d);" % (opcode.r1, opcode.r2))
 			elif opcode.arg == "enumerate":
 				make_scriptlet("r%d = com.livinglogic.ul4.Utils.enumerate(r%d);" % (opcode.r1, opcode.r2))
 			elif opcode.arg == "isnone":
-				make_scriptlet("r%d = (r%d == null) ? Boolean.TRUE : Boolean.FALSE;" % (opcode.r1, opcode.r2))
+				make_scriptlet("r%d = (r%d == null);" % (opcode.r1, opcode.r2))
 			elif opcode.arg == "isstr":
-				make_scriptlet("r%d = ((r%d != null) && (r%d instanceof String)) ? Boolean.TRUE : Boolean.FALSE;" % (opcode.r1, opcode.r2, opcode.r2))
+				make_scriptlet("r%d = ((r%d != null) && (r%d instanceof String));" % (opcode.r1, opcode.r2, opcode.r2))
 			elif opcode.arg == "isint":
-				make_scriptlet("r%d = ((r%d != null) && (r%d instanceof Integer)) ? Boolean.TRUE : Boolean.FALSE;" % (opcode.r1, opcode.r2, opcode.r2))
+				make_scriptlet("r%d = ((r%d != null) && (r%d instanceof Integer));" % (opcode.r1, opcode.r2, opcode.r2))
 			elif opcode.arg == "isfloat":
-				make_scriptlet("r%d = ((r%d != null) && (r%d instanceof Double)) ? Boolean.TRUE : Boolean.FALSE;" % (opcode.r1, opcode.r2, opcode.r2))
+				make_scriptlet("r%d = ((r%d != null) && (r%d instanceof Double));" % (opcode.r1, opcode.r2, opcode.r2))
 			elif opcode.arg == "isbool":
-				make_scriptlet("r%d = ((r%d != null) && (r%d instanceof Boolean)) ? Boolean.TRUE : Boolean.FALSE;" % (opcode.r1, opcode.r2, opcode.r2))
+				make_scriptlet("r%d = ((r%d != null) && (r%d instanceof Boolean));" % (opcode.r1, opcode.r2, opcode.r2))
 			elif opcode.arg == "isdate":
-				make_scriptlet("r%d = ((r%d != null) && (r%d instanceof java.util.Date)) ? Boolean.TRUE : Boolean.FALSE;" % (opcode.r1, opcode.r2, opcode.r2))
+				make_scriptlet("r%d = ((r%d != null) && (r%d instanceof java.util.Date));" % (opcode.r1, opcode.r2, opcode.r2))
 			elif opcode.arg == "islist":
-				make_scriptlet("r%d = ((r%d != null) && (r%d instanceof java.util.List)) ? Boolean.TRUE : Boolean.FALSE;" % (opcode.r1, opcode.r2, opcode.r2))
+				make_scriptlet("r%d = ((r%d != null) && (r%d instanceof java.util.List));" % (opcode.r1, opcode.r2, opcode.r2))
 			elif opcode.arg == "isdict":
-				make_scriptlet("r%d = ((r%d != null) && (r%d instanceof java.util.Map)) ? Boolean.TRUE : Boolean.FALSE;" % (opcode.r1, opcode.r2, opcode.r2))
+				make_scriptlet("r%d = ((r%d != null) && (r%d instanceof java.util.Map));" % (opcode.r1, opcode.r2, opcode.r2))
 			elif opcode.arg == "istemplate":
-				make_scriptlet("r%d = ((r%d != null) && (r%d instanceof .livinglogic.ul4.Template)) ? Boolean.TRUE : Boolean.FALSE;" % (opcode.r1, opcode.r2, opcode.r2))
+				make_scriptlet("r%d = ((r%d != null) && (r%d instanceof com.livinglogic.ul4.Template));" % (opcode.r1, opcode.r2, opcode.r2))
+			elif opcode.arg == "iscolor":
+				make_scriptlet("r%d = ((r%d != null) && (r%d instanceof com.livinglogic.ul4.Color));" % (opcode.r1, opcode.r2, opcode.r2))
 			elif opcode.arg == "chr":
 				make_scriptlet("r%d = com.livinglogic.ul4.Utils.chr(r%d);" % (opcode.r1, opcode.r2))
 			elif opcode.arg == "ord":
@@ -397,6 +420,8 @@ def fromul4(template, variables="variables", indent=0):
 				make_scriptlet("r%d = %s.get(r%d);" % (opcode.r1, variables, opcode.r2))
 			elif opcode.arg == "json":
 				make_scriptlet("r%d = com.livinglogic.ul4.Utils.json(r%d);" % (opcode.r1, opcode.r2))
+			elif opcode.arg == "reversed":
+				make_scriptlet("r%d = com.livinglogic.ul4.Utils.reversed(r%d);" % (opcode.r1, opcode.r2))
 			else:
 				raise ul4c.UnknownFunctionError(opcode.arg)
 		elif opcode.code == "callfunc2":
@@ -489,7 +514,7 @@ def fromul4(template, variables="variables", indent=0):
 			elif opcode.arg == "rfind":
 				make_scriptlet("r%d = com.livinglogic.ul4.Utils.rfind(r%d, r%d);" % (opcode.r1, opcode.r2, opcode.r3))
 			elif opcode.arg == "format":
-				make_scriptlet("r%d = com.livinglogic.ul4.Utils.format(r%d, r%d, defaultLocale);" % (opcode.r1, opcode.r2, opcode.r3))
+				make_scriptlet("r%d = com.livinglogic.ul4.Utils.format(r%d, r%d);" % (opcode.r1, opcode.r2, opcode.r3))
 			elif opcode.arg == "get":
 				make_scriptlet("r%d = ((java.util.Map)r%d).get(r%d);" % (opcode.r1, opcode.r2, opcode.r3))
 			elif opcode.arg == "withlum":
