@@ -1,11 +1,11 @@
 # -*- coding: utf-8 -*-
 
-## Copyright 1999-2008 by LivingLogic AG, Bayreuth/Germany
-## Copyright 1999-2008 by Walter Dörwald
+## Copyright 1999-2009 by LivingLogic AG, Bayreuth/Germany
+## Copyright 1999-2009 by Walter Dörwald
 ##
 ## All Rights Reserved
 ##
-## See xist/__init__.py for the license
+## See ll/__init__.py for the license
 
 
 """
@@ -27,23 +27,6 @@ from ll.xist import xsc
 __docformat__ = "reStructuredText"
 
 
-class CSSWeight(tuple):
-	"""
-	The specificity of a CSS selector as a 3-item tuple as specified by CSS3__.
-	
-	__ http://www.w3.org/TR/css3-selectors/#specificity
-	"""
-
-	def __new__(cls, a=0, b=0, c=0, d=0):
-		return tuple.__new__(cls, (a, b, c, d))
-
-	def __add__(self, other):
-		return CSSWeight(self[0]+other[0], self[1]+other[1], self[2]+other[2], self[3]+other[3])
-
-	def __repr__(self):
-		return "CSSWeight(%r, %r, %r, %r)" % (self[0], self[1], self[2], self[3])
-
-
 class Selector(xsc.WalkFilter):
 	"""
 	Base class for all tree traversal filters that visit the complete tree.
@@ -105,12 +88,6 @@ class Selector(xsc.WalkFilter):
 		"""
 		return NotCombinator(self)
 
-	def cssweight(self):
-		"""
-		Return the CSS specificity of :var:`self` as a :class:`CSSWeight` object.
-		"""
-		return CSSWeight()
-
 
 class IsInstanceSelector(Selector):
 	"""
@@ -123,13 +100,16 @@ class IsInstanceSelector(Selector):
 		>>> from ll.xist.ns import html
 		>>> doc = parsers.parseurl("http://www.python.org", tidy=True)
 		>>> for node in doc.walknode(html.a):
-		... 	print node.bytes()
+		... 	print node.attrs.href, node.attrs.title
 		... 
-		<a id="logolink" accesskey="1" href="http://www.python.org/"><img src="http://www.python.org/images/python-logo.gif" id="logo" border="0" alt="homepage" /></a>
-		<a accesskey="2" href="http://www.python.org/#left%2dhand%2dnavigation"><img id="skiptonav" src="http://www.python.org/images/trans.gif" border="0" alt="skip to navigation" /></a>
-		<a accesskey="3" href="http://www.python.org/#content%2dbody"><img id="skiptocontent" src="http://www.python.org/images/trans.gif" border="0" alt="skip to content" /></a>
-		<a class="reference" href="http://www.python.org/search">Advanced Search</a>
-		<a title="About The Python Language" href="http://www.python.org/about/">About</a>
+		http://www.python.org/ 
+		http://www.python.org/#left%2Dhand%2Dnavigation 
+		http://www.python.org/#content%2Dbody 
+		http://www.python.org/search 
+		http://www.python.org/about/ About The Python Language
+		http://www.python.org/news/ Major Happenings Within the Python Community
+		http://www.python.org/doc/ Tutorials, Library Reference, C API
+		http://www.python.org/download/ Start Running Python Under Windows, Mac, Linux and Others
 		...
 	"""
 	def __init__(self, *types):
@@ -293,7 +273,7 @@ class onlychild(Selector):
 
 		>>> from ll.xist import parsers, xfind
 		>>> doc = parsers.parseurl("http://www.python.org", tidy=True)
-		>>> for node in doc.walknode(<em>xfind.onlychild & html.a</em>):
+		>>> for node in doc.walknode(xfind.onlychild & html.a):
 		... 	print node.bytes()
 		... 
 		<a accesskey="2" href="http://www.python.org/#left%2dhand%2dnavigation"><img id="skiptonav" alt="skip to navigation" src="http://www.python.org/images/trans.gif" border="0" /></a>
@@ -409,23 +389,25 @@ class hasattr_xml(Selector):
 class attrhasvalue(Selector):
 	"""
 	Selector that selects all element nodes where an attribute with the
-	specified Python name has the specified value. For global attributes the
-	attribute class can be passed. Note that "fancy" attributes (i.e. those
+	specified Python name has one of the specified values. For global attributes
+	the attribute class can be passed. Note that "fancy" attributes (i.e. those
 	containing non-text) will not be considered::
 
 		>>> from ll.xist import parsers, xfind
 		>>> doc = parsers.parseurl("http://www.python.org", tidy=True)
 		>>> for node in doc.walknode(xfind.attrhasvalue("rel", "stylesheet")):
-		... 	print repr(node)
+		... 	print node.attrs.href
 		... 
-		<a media="screen" type="text/css" href="http://www.python.org/styles/screen-switcher-default.css" rel="stylesheet" id="screen-switcher-stylesheet" />
-		<a media="scReen" type="text/css" rel="stylesheet" href="http://www.python.org/styles/netscape4.css" />
-		<a media="print" type="text/css" rel="stylesheet" href="http://www.python.org/styles/print.css" />
+		http://www.python.org/styles/screen-switcher-default.css
+		http://www.python.org/styles/netscape4.css
+		http://www.python.org/styles/print.css
 	"""
 
-	def __init__(self, attrname, attrvalue):
+	def __init__(self, attrname, *attrvalues):
 		self.attrname = attrname
-		self.attrvalue = attrvalue
+		if not attrvalues:
+			raise ValueError("need at least one attribute value")
+		self.attrvalues = attrvalues
 
 	def matchpath(self, path):
 		if path:
@@ -433,11 +415,11 @@ class attrhasvalue(Selector):
 			if isinstance(node, xsc.Element) and node.Attrs.isallowed(self.attrname):
 				attr = node.attrs.get(self.attrname)
 				if not attr.isfancy(): # if there are PIs, say no
-					return unicode(attr) == self.attrvalue
+					return unicode(attr) in self.attrvalues
 		return False
 
 	def __str__(self):
-		return "%s(%r, %r)" % (self.__class__.__name__, self.attrname, self.attrvalue)
+		return "%s(%r, %s)" % (self.__class__.__name__, self.attrname, repr(self.attrvalues)[1:-1])
 
 
 class attrhasvalue_xml(Selector):
@@ -446,9 +428,11 @@ class attrhasvalue_xml(Selector):
 	the specified name is treated as an XML name instead of a Python name.
 	"""
 
-	def __init__(self, attrname, attrvalue):
+	def __init__(self, attrname, *attrvalues):
 		self.attrname = attrname
-		self.attrvalue = attrvalue
+		if not attrvalues:
+			raise ValueError("need at least one attribute value")
+		self.attrvalues = attrvalues
 
 	def matchpath(self, path):
 		if path:
@@ -456,7 +440,7 @@ class attrhasvalue_xml(Selector):
 			if isinstance(node, xsc.Element) and node.Attrs.isallowed_xml(self.attrname):
 				attr = node.attrs.get_xml(self.attrname)
 				if not attr.isfancy(): # if there are PIs, say no
-					return unicode(attr) == self.attrvalue
+					return unicode(attr) in self.attrvalues
 		return False
 
 	def __str__(self):
@@ -466,25 +450,28 @@ class attrhasvalue_xml(Selector):
 class attrcontains(Selector):
 	"""
 	Selector that selects all element nodes where an attribute with the
-	specified Python name contains the specified substring in its value. For
-	global attributes the attribute class can be passed. Note that "fancy"
+	specified Python name contains one of the specified substrings in its value.
+	For global attributes the attribute class can be passed. Note that "fancy"
 	attributes (i.e. those containing non-text) will not be considered::
 
 		>>> from ll.xist import parsers, xfind
 		>>> doc = parsers.parseurl("http://www.python.org", tidy=True)
 		>>> for node in doc.walknode(xfind.attrcontains("rel", "stylesheet")):
-		... \tprint repr(node)
+		... 	print node.attrs.rel, node.attrs.href
 		... 
-		<a type="text/css" id="screen-switcher-stylesheet" media="screen" rel="stylesheet" href="http://www.python.org/styles/screen-switcher-default.css" />
-		<a type="text/css" media="scReen" rel="stylesheet" href="http://www.python.org/styles/netscape4.css" />
-		<a type="text/css" media="print" rel="stylesheet" href="http://www.python.org/styles/print.css" />
-		<a type="text/css" title="large text" media="screen" rel="alternate stylesheet" href="http://www.python.org/styles/largestyles.css" />
-		<a type="text/css" title="default fonts" media="screen" rel="alternate stylesheet" href="http://www.python.org/styles/defaultfonts.css" />
+
+		stylesheet http://www.python.org/styles/screen-switcher-default.css
+		stylesheet http://www.python.org/styles/netscape4.css
+		stylesheet http://www.python.org/styles/print.css
+		alternate stylesheet http://www.python.org/styles/largestyles.css
+		alternate stylesheet http://www.python.org/styles/defaultfonts.css
 	"""
 
-	def __init__(self, attrname, attrvalue):
+	def __init__(self, attrname, *attrvalues):
 		self.attrname = attrname
-		self.attrvalue = attrvalue
+		if not attrvalues:
+			raise ValueError("need at least one attribute value")
+		self.attrvalues = attrvalues
 
 	def matchpath(self, path):
 		if path:
@@ -492,7 +479,7 @@ class attrcontains(Selector):
 			if isinstance(node, xsc.Element) and node.Attrs.isallowed(self.attrname):
 				attr = node.attrs.get(self.attrname)
 				if not attr.isfancy(): # if there are PIs, say no
-					return self.attrvalue in unicode(attr)
+					return any(attrvalue in unicode(attr) for attrvalue in self.attrvalues)
 		return False
 
 	def __str__(self):
@@ -505,9 +492,11 @@ class attrcontains_xml(Selector):
 	the specified name is treated as an XML name instead of a Python name.
 	"""
 
-	def __init__(self, attrname, attrvalue):
+	def __init__(self, attrname, *attrvalues):
 		self.attrname = attrname
-		self.attrvalue = attrvalue
+		if not attrvalues:
+			raise ValueError("need at least one attribute value")
+		self.attrvalues = attrvalues
 
 	def matchpath(self, path):
 		if path:
@@ -515,7 +504,7 @@ class attrcontains_xml(Selector):
 			if isinstance(node, xsc.Element) and node.Attrs.isallowed_xml(self.attrname):
 				attr = node.attrs.get_xml(self.attrname)
 				if not attr.isfancy(): # if there are PIs, say no
-					return self.attrvalue in unicode(attr)
+					return any(attrvalue in unicode(attr) for attrvalue in self.attrvalues)
 		return False
 
 	def __str__(self):
@@ -525,7 +514,7 @@ class attrcontains_xml(Selector):
 class attrstartswith(Selector):
 	"""
 	Selector that selects all element nodes where an attribute with the
-	specified Python name starts with the specified string. For global
+	specified Python name starts with any of the specified strings. For global
 	attributes the attribute class can be passed. Note that "fancy" attributes
 	(i.e. those containing non-text) will not be considered::
 
@@ -538,9 +527,11 @@ class attrstartswith(Selector):
 		<input value="search" class="input-button" id="submit" type="submit" name="submit" />
 	"""
 
-	def __init__(self, attrname, attrvalue):
+	def __init__(self, attrname, *attrvalues):
 		self.attrname = attrname
-		self.attrvalue = attrvalue
+		if not attrvalues:
+			raise ValueError("need at least one attribute value")
+		self.attrvalues = attrvalues
 
 	def matchpath(self, path):
 		if path:
@@ -548,7 +539,7 @@ class attrstartswith(Selector):
 			if isinstance(node, xsc.Element) and node.Attrs.isallowed(self.attrname):
 				attr = node.attrs.get(self.attrname)
 				if not attr.isfancy(): # if there are PIs, say no
-					return unicode(attr).startswith(self.attrvalue)
+					return any(unicode(attr).startswith(attrvalue) for attrvalue in self.attrvalues)
 		return False
 
 	def __str__(self):
@@ -561,9 +552,11 @@ class attrstartswith_xml(Selector):
 	that the specified name is treated as an XML name instead of a Python name.
 	"""
 
-	def __init__(self, attrname, attrvalue):
+	def __init__(self, attrname, *attrvalues):
 		self.attrname = attrname
-		self.attrvalue = attrvalue
+		if not attrvalues:
+			raise ValueError("need at least one attribute value")
+		self.attrvalues = attrvalues
 
 	def matchpath(self, path):
 		if path:
@@ -571,7 +564,7 @@ class attrstartswith_xml(Selector):
 			if isinstance(node, xsc.Element) and node.Attrs.isallowed_xml(self.attrname):
 				attr = node.attrs.get_xml(self.attrname)
 				if not attr.isfancy(): # if there are PIs, say no
-					return unicode(attr).startswith(self.attrvalue)
+					return any(unicode(attr).startswith(attrvalue) for attrvalue in self.attrvalues)
 		return False
 
 	def __str__(self):
@@ -581,25 +574,27 @@ class attrstartswith_xml(Selector):
 class attrendswith(Selector):
 	"""
 	Selector that selects all element nodes where an attribute with the
-	specified Python name ends with the specified string. For global attributes
-	the attribute class can be passed. Note that "fancy" attributes (i.e. those
-	containing non-text) will not be considered::
+	specified Python name ends with one of the specified strings. For global
+	attributes the attribute class can be passed. Note that "fancy" attributes
+	(i.e. those containing non-text) will not be considered::
 
 		>>> from ll.xist import parsers, xfind
 		>>> doc = parsers.parseurl("http://www.python.org", tidy=True)
 		>>> for node in doc.walknode(xfind.attrendswith("href", ".css")):
-		... 	print repr(node)
+		... 	print node.attrs.href
 		... 
-		<a href="http://www.python.org/styles/screen-switcher-default.css" type="text/css" rel="stylesheet" id="screen-switcher-stylesheet" media="screen" />
-		<a type="text/css" rel="stylesheet" href="http://www.python.org/styles/netscape4.css" media="scReen" />
-		<a type="text/css" rel="stylesheet" href="http://www.python.org/styles/print.css" media="print" />
-		<a title="large text" type="text/css" rel="alternate stylesheet" href="http://www.python.org/styles/largestyles.css" media="screen" />
-		<a title="default fonts" type="text/css" rel="alternate stylesheet" href="http://www.python.org/styles/defaultfonts.css" media="screen" />
+		http://www.python.org/styles/screen-switcher-default.css
+		http://www.python.org/styles/netscape4.css
+		http://www.python.org/styles/print.css
+		http://www.python.org/styles/largestyles.css
+		http://www.python.org/styles/defaultfonts.css
 	"""
 
-	def __init__(self, attrname, attrvalue):
+	def __init__(self, attrname, *attrvalues):
 		self.attrname = attrname
-		self.attrvalue = attrvalue
+		if not attrvalues:
+			raise ValueError("need at least one attribute value")
+		self.attrvalues = attrvalues
 
 	def matchpath(self, path):
 		if path:
@@ -607,7 +602,7 @@ class attrendswith(Selector):
 			if isinstance(node, xsc.Element) and node.Attrs.isallowed(self.attrname):
 				attr = node.attrs.get(self.attrname)
 				if not attr.isfancy(): # if there are PIs, say no
-					return unicode(attr).endswith(self.attrvalue)
+					return any(unicode(attr).endswith(attrvalue) for attrvalue in self.attrvalues)
 		return False
 
 	def __str__(self):
@@ -620,9 +615,11 @@ class attrendswith_xml(Selector):
 	the specified name is treated as an XML name instead of a Python name.
 	"""
 
-	def __init__(self, attrname, attrvalue):
+	def __init__(self, attrname, *attrvalues):
 		self.attrname = attrname
-		self.attrvalue = attrvalue
+		if not attrvalues:
+			raise ValueError("need at least one attribute value")
+		self.attrvalues = attrvalues
 
 	def matchpath(self, path):
 		if path:
@@ -630,7 +627,7 @@ class attrendswith_xml(Selector):
 			if isinstance(node, xsc.Element) and node.Attrs.isallowed_xml(self.attrname):
 				attr = node.attrs.get_xml(self.attrname)
 				if not attr.isfancy(): # if there are PIs, say no
-					return unicode(attr).endswith(self.attrvalue)
+					return any(unicode(attr).endswith(attrvalue) for attrvalue in self.attrvalues)
 		return False
 
 	def __str__(self):
@@ -639,8 +636,8 @@ class attrendswith_xml(Selector):
 
 class hasid(Selector):
 	"""
-	Selector that selects all element nodes where the ``id`` attribute has the
-	specified value::
+	Selector that selects all element nodes where the ``id`` attribute has one
+	if the specified values::
 
 		>>> from ll.xist import parsers, xfind
 		>>> doc = parsers.parseurl("http://www.python.org", tidy=True)
@@ -650,8 +647,10 @@ class hasid(Selector):
 		<img src="http://www.python.org/images/python-logo.gif" id="logo" alt="homepage" border="0" />
 	"""
 
-	def __init__(self, id):
-		self.id = id
+	def __init__(self, *ids):
+		if not ids:
+			raise ValueError("need at least one id")
+		self.ids = ids
 
 	def matchpath(self, path):
 		if path:
@@ -659,24 +658,21 @@ class hasid(Selector):
 			if isinstance(node, xsc.Element) and node.Attrs.isallowed_xml("id"):
 				attr = node.attrs.get_xml("id")
 				if not attr.isfancy():
-					return unicode(attr) == self.id
+					return unicode(attr) in self.ids
 		return False
 
 	def __str__(self):
 		return "%s(%r)" % (self.__class__.__name__, self.id)
 
-	def cssweight(self):
-		return CSSWeight(0, 1, 0, 0)
-
 
 class hasclass(Selector):
 	"""
-	Selector that selects all element nodes where the ``class`` attribute has
-	the specified value::
+	Selector that selects all element nodes where the ``class`` attribute contains
+	one of the specified values::
 
 		>>> from ll.xist import parsers, xfind
 		>>> doc = parsers.parseurl("http://www.python.org", tidy=True)
-		>>> for node in doc.walknode(xfind.hasclass("reference")<):
+		>>> for node in doc.walknode(xfind.hasclass("reference")):
 		... 	print node.bytes()
 		... 
 		<a class="reference" href="http://www.python.org/search">Advanced Search</a>
@@ -686,8 +682,10 @@ class hasclass(Selector):
 		...
 	"""
 
-	def __init__(self, classname):
-		self.classname = classname
+	def __init__(self, *classnames):
+		if not classnames:
+			raise ValueError("need at least one classname")
+		self.classnames = classnames
 
 	def matchpath(self, path):
 		if path:
@@ -695,15 +693,12 @@ class hasclass(Selector):
 			if isinstance(node, xsc.Element) and node.Attrs.isallowed_xml("class"):
 				attr = node.attrs.get_xml("class")
 				if not attr.isfancy():
-					return self.classname in unicode(attr).split()
+					return any(classname in unicode(attr).split() for classname in self.classnames)
 		return False
 
 	def __str__(self):
 		return "%s(%r)" % (self.__class__.__name__, self.classname)
 
-	def cssweight(self):
-		return CSSWeight(0, 0, 1, 0)
-
 
 class inattr(Selector):
 	"""
@@ -716,7 +711,7 @@ class inattr(Selector):
 	... 
 	text/html; charset=utf-8
 	content-type
-	python programming language object oriented web free source	
+	python programming language object oriented web free source
 	...
 	"""
 	def matchpath(self, path):
@@ -731,8 +726,8 @@ inattr = inattr()
 
 class Combinator(Selector):
 	"""
-	<p>A :class:`Combinator` is a selector that transforms one or combines
-	two or more other selectors in a certain way.</p>
+	A :class:`Combinator` is a selector that transforms one or combines two or
+	more other selectors in a certain way.
 	"""
 
 
@@ -756,9 +751,6 @@ class BinaryCombinator(Combinator):
 			right = "(%s)" % right
 		return "%s%s%s" % (left, self.symbol, right)
 
-	def cssweight(self):
-		return self.left.cssweight()+self.right.cssweight()
-
 
 class ChildCombinator(BinaryCombinator):
 	"""
@@ -925,9 +917,6 @@ class ChainedCombinator(Combinator):
 			v.append(s)
 		return self.symbol.join(v)
 
-	def cssweight(self):
-		raise TypeError("no weight info for chained combinator")
-
 
 class OrCombinator(ChainedCombinator):
 	"""
@@ -939,18 +928,22 @@ class OrCombinator(ChainedCombinator):
 		>>> from ll.xist.ns import html
 		>>> doc = parsers.parseurl("http://www.python.org", tidy=True)
 		>>> for node in doc.walknode(xfind.hasattr("href") | xfind.hasattr("src")):
-		... 	print node.bytes()
+		... 	print node.attrs.href if "href" in node.Attrs else node.attrs.src
 		... 
-		<a type="application/rss+xml" title="RSS" rel="alternate" href="http://www.python.org/channews.rdf" />
-		<a media="screen" type="text/css" id="screen-switcher-stylesheet" rel="stylesheet" href="http://www.python.org/styles/screen-switcher-default.css" />
-		<a media="scReen" type="text/css" rel="stylesheet" href="http://www.python.org/styles/netscape4.css" />
-		<a media="print" type="text/css" rel="stylesheet" href="http://www.python.org/styles/print.css" />
-		<a media="screen" type="text/css" title="large text" rel="alternate stylesheet" href="http://www.python.org/styles/largestyles.css" />
-		<a media="screen" type="text/css" title="default fonts" rel="alternate stylesheet" href="http://www.python.org/styles/defaultfonts.css" />
-		<script src="http://www.python.org/js/iotbs2-key-directors-load.js" type="text/javascript"></script>
-		<script src="http://www.python.org/js/iotbs2-directors.js" type="text/javascript"></script>
-		<script src="http://www.python.org/js/iotbs2-core.js" type="text/javascript"></script>
-		<a accesskey="1" id="logolink" href="http://www.python.org/"><img alt="homepage" src="http://www.python.org/images/python-logo.gif" id="logo" border="0" /></a>
+		http://www.python.org/channews.rdf
+		http://aspn.activestate.com/ASPN/Cookbook/Python/index_rss
+		http://python-groups.blogspot.com/feeds/posts/default
+		http://www.showmedo.com/latestVideoFeed/rss2.0?tag=python
+		http://www.awaretek.com/python/index.xml
+		http://pyfound.blogspot.com/feeds/posts/default
+		http://www.python.org/dev/peps/peps.rss
+		http://www.python.org/community/jobs/jobs.rss
+		http://www.reddit.com/r/Python/.rss
+		http://www.python.org/styles/screen-switcher-default.css
+		http://www.python.org/styles/netscape4.css
+		http://www.python.org/styles/print.css
+		http://www.python.org/styles/largestyles.css
+		http://www.python.org/styles/defaultfonts.css
 		...
 	"""
 
@@ -1093,11 +1086,11 @@ class nthchild(Selector):
 
 class nthoftype(Selector):
 	"""
-	An :class:`nthchild` object is a selector that selects every node that is
+	An :class:`nthoftype` object is a selector that selects every node that is
 	the n-th node of a specified type among its siblings. Similar to
 	:class:`nthchild` :class:`nthoftype` supports negative and positive indices
 	as well as ``"even"`` and ``"odd"``. Which types are checked can be passed
-	explicitely. If no types are passed the type of the node itself is used::
+	explicitly. If no types are passed the type of the node itself is used::
 
 		>>> from ll.xist import parsers, xfind
 		>>> from ll.xist.ns import html
