@@ -13,7 +13,7 @@ import re, datetime, StringIO
 
 import py.test
 
-from ll import ul4c
+from ll import ul4c, color
 
 
 def check(result, source, **variables):
@@ -585,16 +585,28 @@ def test_function_str():
 
 def test_function_int():
 	checkrunerror("function u?'int' unknown", u"<?print int()?>")
-	checkrunerror("function u?'int' unknown", u"<?print int(1, 2)?>")
+	checkrunerror("function u?'int' unknown", u"<?print int(1, 2, 3)?>")
 	checkrunerror("int\\(\\) argument must be a string or a number, not 'NoneType'", u"<?print int(data)?>", data=None)
 	check("1", u"<?print int(data)?>", data=True)
 	check("0", u"<?print int(data)?>", data=False)
 	check("42", u"<?print int(data)?>", data=42)
 	check("4", u"<?print int(data)?>", data=4.2)
 	check("42", u"<?print int(data)?>", data="42")
+	check("66", u"<?print int(data, 16)?>", data="42")
 	checkrunerror("invalid literal for int\\(\\) with base 10: 'foo'", u"<?print int(data)?>", data="foo")
 
 
+def test_function_float():
+	checkrunerror("function u?'float' unknown", u"<?print float()?>")
+	checkrunerror("function u?'float' unknown", u"<?print float(1, 2, 3)?>")
+	checkrunerror("float\\(\\) argument must be a string or a number", u"<?print float(data)?>", data=None)
+	check("1.0", u"<?print float(data)?>", data=True)
+	check("0.0", u"<?print float(data)?>", data=False)
+	check("42.0", u"<?print float(data)?>", data=42)
+	check("4.2", u"<?print float(data)?>", data=4.2)
+	check("42.0", u"<?print float(data)?>", data="42")
+
+
 def test_function_len():
 	checkrunerror("function u?'len' unknown", u"<?print len()?>")
 	checkrunerror("function u?'len' unknown", u"<?print len(1, 2)?>")
@@ -637,6 +649,7 @@ def test_function_isnone():
 	check("False", code, data=[])
 	check("False", code, data={})
 	check("False", code, data=ul4c.compile(u""))
+	check("False", code, data=color.red)
 
 
 def test_function_isbool():
@@ -654,6 +667,7 @@ def test_function_isbool():
 	check("False", code, data=[])
 	check("False", code, data={})
 	check("False", code, data=ul4c.compile(u""))
+	check("False", code, data=color.red)
 
 
 def test_function_isint():
@@ -671,6 +685,7 @@ def test_function_isint():
 	check("False", code, data=[])
 	check("False", code, data={})
 	check("False", code, data=ul4c.compile(u""))
+	check("False", code, data=color.red)
 
 
 def test_function_isfloat():
@@ -688,6 +703,7 @@ def test_function_isfloat():
 	check("False", code, data=[])
 	check("False", code, data={})
 	check("False", code, data=ul4c.compile(u""))
+	check("False", code, data=color.red)
 
 
 def test_function_isstr():
@@ -705,6 +721,7 @@ def test_function_isstr():
 	check("False", code, data=[])
 	check("False", code, data={})
 	check("False", code, data=ul4c.compile(u""))
+	check("False", code, data=color.red)
 
 
 def test_function_isdate():
@@ -722,6 +739,7 @@ def test_function_isdate():
 	check("False", code, data=[])
 	check("False", code, data={})
 	check("False", code, data=ul4c.compile(u""))
+	check("False", code, data=color.red)
 
 
 def test_function_islist():
@@ -739,6 +757,7 @@ def test_function_islist():
 	check("True", code, data=[])
 	check("False", code, data={})
 	check("False", code, data=ul4c.compile(u""))
+	check("False", code, data=color.red)
 
 
 def test_function_isdict():
@@ -756,6 +775,7 @@ def test_function_isdict():
 	check("False", code, data=[])
 	check("True", code, data={})
 	check("False", code, data=ul4c.compile(u""))
+	check("False", code, data=color.red)
 
 
 def test_function_istemplate():
@@ -773,6 +793,25 @@ def test_function_istemplate():
 	check("False", code, data=[])
 	check("False", code, data={})
 	check("True", code, data=ul4c.compile(u""))
+	check("False", code, data=color.red)
+
+
+def test_function_iscolor():
+	checkrunerror("function u?'iscolor' unknown", u"<?print iscolor()?>")
+	checkrunerror("function u?'iscolor' unknown", u"<?print iscolor(1, 2)?>")
+	code = u"<?print iscolor(data)?>"
+	check("False", code, data=None)
+	check("False", code, data=True)
+	check("False", code, data=False)
+	check("False", code, data=42)
+	check("False", code, data=4.2)
+	check("False", code, data="foo")
+	check("False", code, data=datetime.datetime.now())
+	check("False", code, data=())
+	check("False", code, data=[])
+	check("False", code, data={})
+	check("False", code, data=ul4c.compile(u""))
+	check("True", code, data=color.red)
 
 
 def test_function_get():
@@ -904,9 +943,19 @@ def test_function_type():
 	check("list", code, x=[1, 2])
 	check("dict", code, x={1: 2})
 	check("template", code, x=ul4c.compile(""))
+	check("color", code, x=color.red)
 	check("", code, x=1j)
 
 
+def test_function_reversed():
+	checkrunerror("function u?'reversed' unknown", u"<?print reversed()?>")
+	checkrunerror("function u?'reversed' unknown", u"<?print reversed(1, 2)?>")
+	code = u"<?for i in reversed(x)?>(<?print i?>)<?end for?>"
+	check("(3)(2)(1)", code, x="123")
+	check("(3)(2)(1)", code, x=[1, 2, 3])
+	check("(3)(2)(1)", code, x=(1, 2, 3))
+
+
 def test_method_upper():
 	check("GURK", u"<?print 'gurk'.upper()?>")
 
@@ -962,6 +1011,15 @@ def test_method_replace():
 	check('goork', ur"<?print 'gurk'.replace('u', 'oo')?>")
 
 
+def test_method_render():
+	t = ul4c.compile(u'(<?print data?>)')
+	check('(GURK)', u"<?print t.render(data='gurk').upper()?>", t=t)
+	check('(GURK)', u"<?print t.render(**{'data': 'gurk'}).upper()?>", t=t)
+
+	t = ul4c.compile(u'(gurk)')
+	check('(GURK)', u"<?print t.render().upper()?>", t=t)
+
+
 def test_method_format():
 	now = datetime.datetime.now()
 	format = "%Y-%m-%d %H:%M:%S"
@@ -1026,6 +1084,11 @@ def test_method_witha():
 	check('#0063a82a', u'<?print repr(#0063a8.witha(42))?>')
 
 
+def test_method_join():
+	check('1,2,3,4', u'<?print ",".join("1234")?>')
+	check('1,2,3,4', u'<?print ",".join([1, 2, 3, 4])?>')
+
+
 def test_render():
 	t = ul4c.compile(u'<?print prefix?><?print data?><?print suffix?>')
 	check('(f)(o)(o)', u'<?for c in data?><?render t(data=c, prefix="(", suffix=")")?><?end for?>', t=t, data='foo')
@@ -1037,6 +1100,10 @@ def test_render_var():
 	check('42,43,42', u'<?print x?>,<?render t(x=x)?>,<?print x?>', t=t, x=42)
 
 
+def test_def():
+	check('foo', u'<?def lower?><?print x.lower()?><?end def?><?print lower.render(x="FOO")?>')
+
+
 def test_parse():
 	check('42', u'<?print data.Noner?>', data=dict(Noner=42))
 
@@ -1110,6 +1177,8 @@ def universaltemplate():
 		<?print x.find(1, 2, 3)?>
 		<?if x?>gurk<?elif y?>hurz<?else?>hinz<?end if?>
 		<?render x(a=1, b=2)?>
+		<?def x?>foo<?end def?>
+		<?render x()?>
 	""")
 
 
